# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.parse_error import ParsingError
from ....core.request_options import RequestOptions
from ....core.serialization import convert_and_respect_annotation_metadata
from ....core.unchecked_base_model import construct_type
from ....errors.unprocessable_entity_error import UnprocessableEntityError
from ....types.column_filter import ColumnFilter
from ....types.workspace_analytics_query_response_model import WorkspaceAnalyticsQueryResponseModel
from .types.body_list_api_requests_v_1_workspace_analytics_requests_post_sort import (
    BodyListApiRequestsV1WorkspaceAnalyticsRequestsPostSort,
)
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRequestsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        *,
        start_time: typing.Optional[int] = OMIT,
        end_time: typing.Optional[int] = OMIT,
        limit: typing.Optional[int] = OMIT,
        sort: typing.Optional[BodyListApiRequestsV1WorkspaceAnalyticsRequestsPostSort] = OMIT,
        filters: typing.Optional[typing.Sequence[ColumnFilter]] = OMIT,
        search: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[WorkspaceAnalyticsQueryResponseModel]:
        """
        Returns a list of API requests. Supports filtering by time range, column filters, and search terms. At least one of start_time or end_time must be provided. An optional sort parameter controls timestamp ordering. Results are ordered by timestamp. Descending if end_time is used, ascending if start_time is used. The response is a tabular structure with columns, column_types, column_units, and rows.

        Parameters
        ----------
        start_time : typing.Optional[int]
            Start of the time range as a Unix timestamp in milliseconds.

        end_time : typing.Optional[int]
            End of the time range as a Unix timestamp in milliseconds.

        limit : typing.Optional[int]

        sort : typing.Optional[BodyListApiRequestsV1WorkspaceAnalyticsRequestsPostSort]
            Optional timestamp sort direction. If omitted, defaults to desc when end_time is provided, otherwise asc.

        filters : typing.Optional[typing.Sequence[ColumnFilter]]

        search : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WorkspaceAnalyticsQueryResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/workspace/analytics/requests",
            method="POST",
            json={
                "start_time": start_time,
                "end_time": end_time,
                "limit": limit,
                "sort": sort,
                "filters": convert_and_respect_annotation_metadata(
                    object_=filters, annotation=typing.Sequence[ColumnFilter], direction="write"
                ),
                "search": search,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WorkspaceAnalyticsQueryResponseModel,
                    construct_type(
                        type_=WorkspaceAnalyticsQueryResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRequestsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        *,
        start_time: typing.Optional[int] = OMIT,
        end_time: typing.Optional[int] = OMIT,
        limit: typing.Optional[int] = OMIT,
        sort: typing.Optional[BodyListApiRequestsV1WorkspaceAnalyticsRequestsPostSort] = OMIT,
        filters: typing.Optional[typing.Sequence[ColumnFilter]] = OMIT,
        search: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[WorkspaceAnalyticsQueryResponseModel]:
        """
        Returns a list of API requests. Supports filtering by time range, column filters, and search terms. At least one of start_time or end_time must be provided. An optional sort parameter controls timestamp ordering. Results are ordered by timestamp. Descending if end_time is used, ascending if start_time is used. The response is a tabular structure with columns, column_types, column_units, and rows.

        Parameters
        ----------
        start_time : typing.Optional[int]
            Start of the time range as a Unix timestamp in milliseconds.

        end_time : typing.Optional[int]
            End of the time range as a Unix timestamp in milliseconds.

        limit : typing.Optional[int]

        sort : typing.Optional[BodyListApiRequestsV1WorkspaceAnalyticsRequestsPostSort]
            Optional timestamp sort direction. If omitted, defaults to desc when end_time is provided, otherwise asc.

        filters : typing.Optional[typing.Sequence[ColumnFilter]]

        search : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WorkspaceAnalyticsQueryResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/workspace/analytics/requests",
            method="POST",
            json={
                "start_time": start_time,
                "end_time": end_time,
                "limit": limit,
                "sort": sort,
                "filters": convert_and_respect_annotation_metadata(
                    object_=filters, annotation=typing.Sequence[ColumnFilter], direction="write"
                ),
                "search": search,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WorkspaceAnalyticsQueryResponseModel,
                    construct_type(
                        type_=WorkspaceAnalyticsQueryResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
